/****************************************************************************************
*                               TERMS OF USE: MIT License                               *
*                                                                                       *
* Permission is hereby granted, free of charge, to any person obtaining a copy of this  *
* software and associated documentation files (the "Software"), to deal in the Software *
* without restriction, including without limitation the rights to use, copy, modify,    *
* merge, publish, distribute, sublicense, and/or sell copies of the Software, and to    *
* permit persons to whom the Software is furnished to do so, subject to the following   *
* conditions:                                                                           *
*                                                                                       *
* The above copyright notice and this permission notice shall be included in all copies *
* or substantial portions of the Software.                                              *
*                                                                                       *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,   *
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A         *
* PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT    *
* HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF  *
* CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE  *
* OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                         *
****************************************************************************************/

#include <propeller.h>
#include "ffds1.h"

static ffds1 thisobj;

extern void fds_entry(void) asm("ffds1.entry");
extern int *period_ptr asm("ffds1.period_ptr");
extern int maskRX asm("ffds1.maskRX");
extern int maskTX asm("ffds1.maskTX");
volatile uint8_t *rx_head_ptr asm("ffds1.rx_head_ptr");
volatile uint8_t *rx_end_ptr asm("ffds1.rx_end_ptr");
extern int ctra_val asm("ffds1.ctra_val");
extern int ctrb_val asm("ffds1.ctrb_val");
volatile uint16_t *update_head_ptr asm("ffds1.update_head_ptr");

#ifdef TEST
int main()
{
}
#endif

int ffds1_start(int rx_pin, int tx_pin, int rate)
{
        // start by stopping, just in case we were already running
        ffds1_stop();
        // set up TX
        maskTX = ctra_val = 0;
    if (tx_pin >= 0) {
        maskTX = ((1<<tx_pin));
        // use counter A to output bits (NCO single-ended)...if desired
        ctra_val = (0x10000000 | tx_pin);
    }
    // set up RX
    maskRX = (ctrb_val = 0);
    if (rx_pin >= 0) {
        maskRX = ((1<<rx_pin));
        // use counter B (in LOGIC !A) to see if I need to use the 1st or 2nd sample for RX
        ctrb_val = (0x54000000 | rx_pin);
    }
    // timing
    ffds1_setbaud(rate);
    period_ptr = &thisobj.Half_bit_period;
    // set up the buffers and pointers
    memset((uint8_t*)thisobj.Rx_buffer, 0, (sizeof thisobj.Rx_buffer));
    rx_head_ptr = thisobj.Rx_buffer;
    rx_end_ptr = thisobj.Rx_buffer + (sizeof thisobj.Rx_buffer);
    thisobj.Rx_head = 0;
    thisobj.Rx_tail = 0;
    update_head_ptr = &thisobj.Rx_head;
    // start the new cog; Value here doesn't matter, just not NULL.
    thisobj.Write_buf_ptr = thisobj.Rx_buffer;
    thisobj.Cog = 1 + coginit(((int)thisobj.Write_buf_ptr)>>2, ((int)(&fds_entry))>>2, 8);
    if (thisobj.Cog) {
        while (thisobj.Write_buf_ptr)
            ;
        return -1;
    }
    return 0;
}

void ffds1_stop(void)
{
    if (thisobj.Cog) {
        cogstop(thisobj.Cog - 1);
        thisobj.Cog = 0;
    }
}

int ffds1_setbaud(int Rate)
{
    int got_rate = 0;
    got_rate = ffds1_setbaudclock(Rate, CLKFREQ);
    return got_rate;
}

int ffds1_setbaudclock(int Rate, int Sysclock)
{
    int got_rate = 0;
    got_rate = (((Sysclock >> 1) + (Rate >> 1)) / Rate);
    thisobj.Half_bit_period = (got_rate > Min_half_period) ? got_rate : Min_half_period;
    got_rate = -(got_rate >= Min_half_period);
    return got_rate;
}

void ffds1_str(char *buf)
{
    if (buf[0]) {
        buf += (strlen(buf)-1) << 16;
        thisobj.Write_buf_ptr = buf;
        while (thisobj.Write_buf_ptr)
            ;
    }
}

void ffds1_tx(char c)
{
    thisobj.Send_temp = c;
    thisobj.Write_buf_ptr = &thisobj.Send_temp;
    while (thisobj.Write_buf_ptr)
        ;
}

void ffds1_txbuf(char *buf, size_t count)
{
    if (count > 0) {
        buf += (count-1) << 16;
        thisobj.Write_buf_ptr = buf;
        while (thisobj.Write_buf_ptr)
            ;
    }
}

void ffds1_txbufnowait(char *buf, size_t count)
{
    if (count > 0) {
        buf += (count - 1) << 16;
        thisobj.Write_buf_ptr = buf;
    }
}

void ffds1_waitfortx(void)
{
    while (thisobj.Write_buf_ptr)
        ;
}

void ffds1_rxflush(void)
{
    thisobj.Rx_tail = thisobj.Rx_head;
}

int ffds1_rxcheck(void)
{
    int Rxbyte = -1;

    if (thisobj.Rx_tail != thisobj.Rx_head) {
        Rxbyte = thisobj.Rx_buffer[thisobj.Rx_tail];
        thisobj.Rx_buffer[thisobj.Rx_tail] = 0;
        thisobj.Rx_tail = ((thisobj.Rx_tail + 1) % (sizeof thisobj.Rx_buffer));
    }
    return Rxbyte;
}

int ffds1_rxtime(int Ms)
{
    int Rxbyte = 0;
    int Tout = (((CLKFREQ / 1000) * Ms) + CNT);
    while (Tout < CNT) {
        if ((Rxbyte = ffds1_rxcheck()) >= 0)
            return Rxbyte;
    }
    return -1;
}

char ffds1_rx(void)
{
    int val;
    while ((val = ffds1_rxcheck()) < 0)
        ;
    return val;
}

void ffds1_dec(int Value)
{
    int hadADigitYet = 0;

    if (Value < 0) {
        Value = (-Value);
        ffds1_tx('-');
    }

    int i = 1000000000;
    int digits = 10;
    while (digits--) {
        if (Value >= i) {
            ffds1_tx((Value / i) + '0');
            Value %= i;
            hadADigitYet--;
        } else {
            if ((hadADigitYet) || (i == 1)) {
                ffds1_tx('0');
            }
        }
        i /= 10;
    }
}

void ffds1_hex(int Value, int Digits)
{ // Fixed
    static char hexlookup[] = "0123456789ABCDEF";
    Value <<= ((8 - Digits) << 2);

    while (Digits--) {
      ffds1_tx(hexlookup[(Value & 0xf0000000) >> 28]);
      Value <<= 4;
    }
}

void ffds1_bin(int Value, int Digits)
{ // Fixed
    Value <<= (32 - Digits);
    while (Digits--) {
        ffds1_tx((Value & 0x80000000) ? '1' : '0');
        Value <<= 1;
    }
}
